from admyral.actions.integrations.cdr.ms_defender_for_cloud import (
    list_ms_defender_for_cloud_alerts,
)
from admyral.actions.integrations.cdr.wiz import list_wiz_alerts

__all__ = ["list_ms_defender_for_cloud_alerts", "list_wiz_alerts"]
